@@ -4,6 +4,9 @@ pub(crate) mod buf;
 pub mod extensions;
 /// Common future types used by `hrpc-rs`.
 pub mod future;
+/// Common layers that can be used in both clients and servers.
+#[cfg(feature = "_common")]
+pub mod layer;
 /// Common code to work with sockets.
 #[cfg(feature = "_common")]
 pub mod socket;
